package jp.programmingmat.www.atfskscroll;

import android.content.Context;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Bitmap;
import android.graphics.Rect;

/*
	マップ描画ビュー
*/
public class appView extends SurfaceView implements SurfaceHolder.Callback, Runnable {

	private Bitmap bmp_chips;

	private int chip_width, chip_height;
	private int chip_cols, chip_rows;
	private int map_cols, map_rows;

	private int x, y;
	private int v_x, v_y;
	private int v_dx, v_dy;

	private int scroll_dir, scroll_pos, scroll_pixel;

	private int[] mapData;

	private Thread thread;
	private SurfaceHolder _holder;

	private volatile boolean _alive;

	private SoftPad _pad;

	private long process_default_ns = 10 * 1000000;

	private long frame_ns = 0;
	private long frame_ns_sum = 0;
	private long frame_ms_disp = 0;

	private Paint disp_paint;

	public appView(Context context, SoftPad pad) {

		super(context);

		_holder = getHolder();

		_holder.addCallback(this);

		_pad = pad;

		// 処理時間表示用paint作成
		disp_paint = new Paint();

		disp_paint.setTextSize(18);

	}

	public void setChipBitmap(Bitmap bmp, int cwidth, int cheight) {

		bmp_chips = Bitmap.createBitmap(bmp);

		chip_width = cwidth;
		chip_height = cheight;

		// 画面上に表示するマップチップ数を設定
		chip_cols = 11;
		chip_rows = 11;

		// マップサイズ設定
		map_cols = 30;
		map_rows = 30;

		// マップデータ配列を作成
		mapData = new int[map_cols * map_rows];

		// マップデータを作成
		for (int i = 0;i < mapData.length;i++) {
			mapData[i] = (int)(Math.random() * 16);
		}

		// マップ上の初期位置設定
		x = 10;
		y = 10;

		v_x = x;
		v_y = y;

		v_dx = 0;
		v_dy = 0;

		// スクロール単位を1ピクセルに設定
		scroll_pixel = 1;

	}

	public void surfaceChanged (SurfaceHolder holder, int format, int width, int height) {
	}

	public void surfaceCreated(SurfaceHolder holder) {

		// メインループ実行フラグをセット
		_alive = true;

		// メインループ用スレッド作成
		thread = new Thread(this);
		thread.start();

	}

	public void surfaceDestroyed(SurfaceHolder holder) {

		// メインループ実行フラグをリセット
		_alive = false;

	}

	public void run() {

		int process_count = 0;
		long process_prev_ns = 0;
		long process_ns = 0;

		long frame_prev_ns = 0;

		// メインループ
		while (_alive) {

			// 今回の処理開始時刻を取得
			frame_ns = System.nanoTime();

			// 前回処理からの時間を処理時間累計に加算
			frame_ns_sum += frame_ns - frame_prev_ns;

			// 今回の処理開始時刻を記録
			frame_prev_ns = frame_ns;

			// 10回に一度表示用の処理時間を更新
			if (process_count % 10 == 9) {

				// 処理時間の平均をミリ秒単位で算出
				frame_ms_disp = (long)((frame_ns_sum / 10) / 1000000);

				// 処理時間累計をクリア
				frame_ns_sum = 0;

			}

			if (scroll_dir == 0) { // 非スクロール時

				// パッドがタッチされていたらスクロール開始
				switch (_pad.pad_active) {

				case SoftPad.PADACTIVE_UP:

					if (y > 0) {

						scroll_dir = _pad.pad_active;
						scroll_pos = 0;

						v_x = x;
						v_y = y - 1;

						v_dx = 0;
						v_dy = -chip_height;

					}

					break;

				case SoftPad.PADACTIVE_RIGHT:

					if (x < (map_cols - chip_cols)) {

						scroll_dir = _pad.pad_active;
						scroll_pos = 0;

						v_x = x;
						v_y = y;

						v_dx = 0;
						v_dy = 0;

					}

					break;

				case SoftPad.PADACTIVE_DOWN:

					if (y < (map_rows - chip_rows)) {

						scroll_dir = _pad.pad_active;
						scroll_pos = 0;

						v_x = x;
						v_y = y;

						v_dx = 0;
						v_dy = 0;

					}

					break;

				case SoftPad.PADACTIVE_LEFT:

					if (x > 0) {

						scroll_dir = _pad.pad_active;
						scroll_pos = 0;

						v_x = x - 1;
						v_y = y;

						v_dx = -chip_width;
						v_dy = 0;

					}

					break;

				}

			} else { // スクロール中

				switch (scroll_dir) {

				case SoftPad.PADACTIVE_UP:

					v_dy = -chip_height + scroll_pos;

					scroll_pos += scroll_pixel;

					if (scroll_pos >= chip_height) {

						y--;

						v_x = x;
						v_y = y;

						v_dx = 0;
						v_dy = 0;

						scroll_dir = 0;

					}

					break;

				case SoftPad.PADACTIVE_RIGHT:

					v_dx = -scroll_pos;

					scroll_pos += scroll_pixel;

					if (scroll_pos >= chip_width) {

						x++;

						v_x = x;
						v_y = y;

						v_dx = 0;
						v_dy = 0;

						scroll_dir = 0;

					}

					break;

				case SoftPad.PADACTIVE_DOWN:

					v_dy = -scroll_pos;

					scroll_pos += scroll_pixel;

					if (scroll_pos >= chip_height) {

						y++;

						v_x = x;
						v_y = y;

						v_dx = 0;
						v_dy = 0;

						scroll_dir = 0;

					}

					break;

				case SoftPad.PADACTIVE_LEFT:

					v_dx = -chip_width + scroll_pos;

					scroll_pos += scroll_pixel;

					if (scroll_pos >= chip_width) {

						x--;

						v_x = x;
						v_y = y;

						v_dx = 0;
						v_dy = 0;

						scroll_dir = 0;

					}

					break;

				}

			}

			switch ((process_count / 5) % 3) {

			case 0:

				disp_paint.setColor(0xff800000);

				break;

			case 1:

				disp_paint.setColor(0xff008000);

				break;

			case 2:

				disp_paint.setColor(0xff000080);

				break;

			}

			Canvas cv = _holder.lockCanvas();
			onDraw(cv);
			_holder.unlockCanvasAndPost(cv);

			// 前回の処理終了時からの経過時間を算出
			process_ns = System.nanoTime() - process_prev_ns;

			// ウエイトを算出
			long wait_ns = process_default_ns - process_ns;

			long wait_ms = wait_ns / 1000000;

			if (wait_ms < 1) {
				wait_ms = 1;
			}

			// ウエイト
			try {
				Thread.sleep(wait_ms);
			} catch (Exception e) {
				if (_alive) {
					break;
				}
			}

			// 今回の処理終了時間を記録
			process_prev_ns = System.nanoTime();

			process_count++;

		}

	}

	@Override
	protected void onDraw(Canvas cv) {

		Rect src = new Rect();
		Rect dst = new Rect();

		// マップ画面を描画
		for (int i = 0; i < chip_rows;i++) {
			for (int j = 0; j < chip_cols;j++) {

				// 描画位置に対応するマップチップを取得
				int tip = mapData[(j + v_x) + (i + v_y) * chip_cols];

				// チップ画像上の位置を算出
				int chip_x = tip % 4;
				int chip_y = (int)(Math.floor(tip / 4));

				// チップ画像内の転送位置を算出
				src.left = chip_x * chip_width;
				src.top = chip_y * chip_height;
				src.right = src.left + chip_width;
				src.bottom = src.top + chip_height;

				// 描画範囲を算出
				dst.left = j * chip_width + v_dx;
				dst.top = i * chip_height + v_dy;
				dst.right = dst.left + chip_width;
				dst.bottom = dst.top + chip_height;

				// チップ画像を転送
				cv.drawBitmap(bmp_chips, src, dst, null);

			}
		}

		cv.drawText("フレーム処理平均時間:" + frame_ms_disp + "ms", 0, disp_paint.descent() - disp_paint.ascent(), disp_paint);

	}

}